@@ -1,10 +1,12 @@
 """Implement the GeometricProgram class"""
 import sys
-import warnings
+import warnings as pywarnings
 from time import time
 from collections import defaultdict
 import numpy as np
+from ..repr_conventions import lineagestr
 from ..small_classes import CootMatrix, SolverLog, Numbers, FixedScalar
+from ..small_scripts import appendsolwarning, initsolwarning
 from ..keydict import KeyDict
 from ..solution_array import SolutionArray
 from .set import ConstraintSet
@@ -59,17 +61,19 @@ class GeometricProgram:
 
     Examples
     --------
-    >>> gp = gpkit.geometric_program.GeometricProgram(
+    >>> gp = gpkit.constraints.gp.GeometricProgram(
                         # minimize
                         x,
                         [   # subject to
-                            1/x  # <= 1, implicitly
+                            x >= 1,
                         ], {})
     >>> gp.solve()
     """
     _result = solve_log = solver_out = model = v_ss = nu_by_posy = None
+    choicevaridxs = integersolve = None
 
-    def __init__(self, cost, constraints, substitutions, *, checkbounds=True):
+    def __init__(self, cost, constraints, substitutions,
+                 *, checkbounds=True, **_):
         self.cost, self.substitutions = cost, substitutions
         for key, sub in self.substitutions.items():
             if isinstance(sub, FixedScalar):
@@ -78,9 +82,9 @@ class GeometricProgram:
                     sub = sub.to(key.units or "dimensionless").magnitude
                 self.substitutions[key] = sub
             if not isinstance(sub, (Numbers, np.ndarray)):
-                raise ValueError("substitution {%s: %s} has invalid value type"
-                                 " %s." % (key, sub, type(sub)))
-        cost_hmap = cost.hmap.sub(self.substitutions, cost.varkeys)
+                raise TypeError("substitution {%s: %s} has invalid value type"
+                                " %s." % (key, sub, type(sub)))
+        cost_hmap = cost.hmap.sub(self.substitutions, cost.vks)
         if any(c <= 0 for c in cost_hmap.values()):
             raise InvalidPosynomial("a GP's cost must be Posynomial")
         hmapgen = ConstraintSet.as_hmapslt1(constraints, self.substitutions)
@@ -117,18 +121,21 @@ class GeometricProgram:
         return missingbounds
 
     def gen(self):
-        "Generates nomial and solve data (A, p_idxs) from posynomials"
-        # k [posys]: number of monomials (rows of A) present in each constraint
+        """Generates nomial and solve data (A, p_idxs) from posynomials.
+
+        k [posys]: number of monomials (rows of A) present in each constraint
+        m_idxs [mons]: monomial indices of each posynomial
+        p_idxs [mons]: posynomial index of each monomial
+        cs, exps [mons]: coefficient and exponents of each monomial
+        varlocs: {vk: monomial indices of each variables' location}
+        meq_idxs: {all indices of equality mons} and {the first index of each}
+        varidxs: {vk: which column corresponds to it in A}
+        A [mons, vks]: sparse array of each monomials' variables' exponents
+
+        """
         self.k = [len(hmap) for hmap in self.hmaps]
-        # m_idxs [mons]: monomial indices of each posynomial
-        self.m_idxs = []
-        # p_idxs [mons]: posynomial index of each monomial
-        self.p_idxs = []
-        # cs, exps [mons]: coefficient and exponents of each monomial
-        self.cs, self.exps = [], []
-        # varlocs: {vk: monomial indices of each variables' location}
+        self.m_idxs, self.p_idxs, self.cs, self.exps = [], [], [], []
         self.varkeys = self.varlocs = defaultdict(list)
-        # meq_idxs: {all indices of equality mons} and {just the first halves}
         self.meq_idxs = MonoEqualityIndexes()
         m_idx = 0
         row, col, data = [], [], []
@@ -149,13 +156,14 @@ class GeometricProgram:
                 for var in exp:
                     self.varlocs[var].append(m_idx)
                 m_idx += 1
-        self.p_idxs = np.array(self.p_idxs, "int32")  # for later use as array
+        self.p_idxs = np.array(self.p_idxs, "int32")  # to use array equalities
         self.varidxs = {vk: i for i, vk in enumerate(self.varlocs)}
+        self.choicevaridxs = {vk: i for i, vk in enumerate(self.varlocs)
+                              if vk.choices}
         for j, (var, locs) in enumerate(self.varlocs.items()):
             row.extend(locs)
             col.extend([j]*len(locs))
             data.extend(self.exps[i][var] for i in locs)
-        # A [mons, vks]: sparse array of each monomials' variables' exponents
         self.A = CootMatrix(row, col, data)
 
     # pylint: disable=too-many-statements, too-many-locals
@@ -170,71 +178,76 @@ class GeometricProgram:
             If a function, passes that function cs, A, p_idxs, and k.
         verbosity : int (default 1)
             If greater than 0, prints solver name and solve time.
+        gen_result : bool (default True)
+            If True, makes a human-readable SolutionArray from solver output.
         **kwargs :
             Passed to solver constructor and solver function.
 
 
         Returns
         -------
-        result : SolutionArray
+        SolutionArray (or dict if gen_result is False)
         """
         solvername, solverfn = _get_solver(solver, kwargs)
-        solverargs = DEFAULT_SOLVER_KWARGS.get(solvername, {})
-        solverargs.update(kwargs)
-        solver_out = {}
-
         if verbosity > 0:
             print("Using solver '%s'" % solvername)
             print(" for %i free variables" % len(self.varlocs))
             print("  in %i posynomial inequalities." % len(self.k))
+
+        solverargs = DEFAULT_SOLVER_KWARGS.get(solvername, {})
+        solverargs.update(kwargs)
+        if self.choicevaridxs and solvername == "mosek_conif":
+            solverargs["choicevaridxs"] = self.choicevaridxs
+            self.integersolve = True
         starttime = time()
-        infeasibility, original_stdout = None, sys.stdout
+        solver_out, infeasibility, original_stdout = {}, None, sys.stdout
         try:
             sys.stdout = SolverLog(original_stdout, verbosity=verbosity-2)
             solver_out = solverfn(c=self.cs, A=self.A, meq_idxs=self.meq_idxs,
                                   k=self.k, p_idxs=self.p_idxs, **solverargs)
         except Infeasible as e:
             infeasibility = e
+        except InvalidLicense as e:
+            raise InvalidLicense("license for solver \"%s\" is invalid."
+                                 % solvername) from e
         except Exception as e:
-            if isinstance(e, InvalidLicense):
-                raise InvalidLicense("license for solver \"%s\" is invalid."
-                                     % solvername) from e
             raise UnknownInfeasible("Something unexpected went wrong.") from e
         finally:
             self.solve_log = "\n".join(sys.stdout)
             sys.stdout = original_stdout
             self.solver_out = solver_out
-            solver_out["solver"] = solvername
-            solver_out["soltime"] = time() - starttime
-            if verbosity > 0:
-                print("Solving took %.3g seconds." % solver_out["soltime"])
+
+        solver_out["solver"] = solvername
+        solver_out["soltime"] = time() - starttime
+        if verbosity > 0:
+            print("Solving took %.3g seconds." % solver_out["soltime"])
 
         if infeasibility:
             if isinstance(infeasibility, PrimalInfeasible):
-                msg = ("The model had no feasible points; "
-                       "you may wish to relax some constraints or constants.")
+                msg = ("The model had no feasible points; relaxing some"
+                       " constraints or constants will probably fix this.")
             elif isinstance(infeasibility, DualInfeasible):
-                msg = ("The model ran to an infinitely low cost;"
-                       " bounding the right variables would prevent this.")
+                msg = ("The model ran to an infinitely low cost"
+                       " (or was otherwise dual infeasible); bounding"
+                       " the right variables will probably fix this.")
             elif isinstance(infeasibility, UnknownInfeasible):
-                msg = "The solver failed for an unknown reason."
-            if verbosity > 0 and solver_out["soltime"] < 1:
-                print(msg + "\nSince this model solved in less than a second,"
-                      " let's run `.debug()` automatically to check.\n`")
+                msg = ("Solver failed for an unknown reason. Relaxing"
+                       " constraints/constants, bounding variables, or"
+                       " using a different solver might fix it.")
+            if verbosity > 0 and solver_out["soltime"] < 1 and self.model:
+                print(msg + "\nSince the model solved in less than a second,"
+                      " let's run `.debug()` to analyze what happened.\n")
                 return self.model.debug(solver=solver)
-            msg += (" Running `.debug()` may pinpoint the trouble. You can"
-                    " also try another solver, or increase the verbosity.")
+            # else, raise a clarifying error
+            msg += (" Running `.debug()` or increasing verbosity may pinpoint"
+                    " the trouble.")
             raise infeasibility.__class__(msg) from infeasibility
 
-        if gen_result:  # NOTE: SIDE EFFECTS
-            self._result = self.generate_result(solver_out,
-                                                verbosity=verbosity-2)
-            return self.result
-        # TODO: remove this "generate_result" closure
-        solver_out["generate_result"] = \
-            lambda: self.generate_result(solver_out, dual_check=False)
-
-        return solver_out
+        if not gen_result:
+            return solver_out
+        # else, generate a human-readable SolutionArray
+        self._result = self.generate_result(solver_out, verbosity=verbosity-2)
+        return self.result
 
     @property
     def result(self):
@@ -255,18 +268,20 @@ class GeometricProgram:
                   ((time() - tic) / soltime * 100))
             tic = time()
         # solution checking #
+        initsolwarning(result, "Solution Inconsistency")
         try:
             tol = SOLUTION_TOL.get(solver_out["solver"], 1e-5)
             self.check_solution(result["cost"], solver_out['primal'],
                                 solver_out["nu"], solver_out["la"], tol)
         except Infeasible as chkerror:
-            chkwarn = str(chkerror)
-            if dual_check or ("Dual" not in chkwarn and "nu" not in chkwarn):
-                print("Solution check warning: %s" % chkwarn)
+            msg = str(chkerror)
+            if not ("Dual" in msg and not dual_check):
+                appendsolwarning(msg, None, result, "Solution Inconsistency")
+                if verbosity > -4:
+                    print("Solution check warning: %s" % msg)
         if verbosity > 0:
             print("Solution checking took %.2g%% of solve time." %
                   ((time() - tic) / soltime * 100))
-            tic = time()
         return result
 
     def _generate_nula(self, solver_out):
@@ -292,24 +307,6 @@ class GeometricProgram:
         return la, nu_by_posy
 
     def _compile_result(self, solver_out):
-        """Creates a result dict (as returned by solve() from solver output
-
-        This internal method is called from within the solve() method, unless
-        solver_out["status"] is not "optimal", in which case a RuntimeWarning
-        is raised prior to this method being called. In that case, users
-        may use this method to attempt to create a results dict from the
-        output of the failed solve.
-
-        Arguments
-        ---------
-        solver_out: dict
-            dict in format returned by solverfn within GeometricProgram.solve
-
-        Returns
-        -------
-        result: dict
-            dict in format returned by GeometricProgram.solve()
-        """
         result = {"cost": float(solver_out["objective"])}
         primal = solver_out["primal"]
         if len(self.varlocs) != len(primal):
@@ -318,40 +315,65 @@ class GeometricProgram:
         result["constants"] = KeyDict(self.substitutions)
         result["variables"] = KeyDict(result["freevariables"])
         result["variables"].update(result["constants"])
+        result["soltime"] = solver_out["soltime"]
+        if self.integersolve:
+            result["choicevariables"] = KeyDict( \
+                {k: v for k, v in result["freevariables"].items()
+                 if k in self.choicevaridxs})
+            result["warnings"] = {"No Dual Solution": [(\
+                "This model has the discretized choice variables"
+                " %s and hence no dual solution. You can fix those variables"
+                " to their optimal value and get sensitivities to the resulting"
+                " continuous problem by updating your model's substitions with"
+                " `sol[\"choicevariables\"]`."
+                % sorted(self.choicevaridxs.keys()), self.choicevaridxs)]}
+            return SolutionArray(result)
+        if self.choicevaridxs:
+            result["warnings"] = {"Freed Choice Variables": [(\
+                "This model has the discretized choice variables"
+                " %s, but since the '%s' solver doesn't support discretization"
+                " they were treated as continuous variables."
+                % (sorted(self.choicevaridxs.keys()), solver_out["solver"]),
+                self.choicevaridxs)]}
+
         result["sensitivities"] = {"constraints": {}}
         la, self.nu_by_posy = self._generate_nula(solver_out)
         cost_senss = sum(nu_i*exp for (nu_i, exp) in zip(self.nu_by_posy[0],
                                                          self.cost.hmap))
-        self.v_ss = cost_senss.copy()
+        gpv_ss = cost_senss.copy()
+        m_senss = defaultdict(float)
         for las, nus, c in zip(la[1:], self.nu_by_posy[1:], self.hmaps[1:]):
             while getattr(c, "parent", None) is not None:
                 c = c.parent
             v_ss, c_senss = c.sens_from_dual(las, nus, result)
             for vk, x in v_ss.items():
-                self.v_ss[vk] = x + self.v_ss.get(vk, 0)
-            while getattr(c, "generated_by", None) is not None:
+                gpv_ss[vk] = x + gpv_ss.get(vk, 0)
+            while getattr(c, "generated_by", None):
                 c = c.generated_by
-            result["sensitivities"]["constraints"][c] = c_senss
+            result["sensitivities"]["constraints"][c] = abs(c_senss)
+            m_senss[lineagestr(c)] += abs(c_senss)
+        # add fixed variables sensitivities to models
+        for vk, senss in gpv_ss.items():
+            m_senss[lineagestr(vk)] += abs(senss)
+        result["sensitivities"]["models"] = dict(m_senss)
         # carry linked sensitivities over to their constants
-        for v in list(v for v in self.v_ss if v.gradients):
-            dlogcost_dlogv = self.v_ss.pop(v)
+        for v in list(v for v in gpv_ss if v.gradients):
+            dlogcost_dlogv = gpv_ss.pop(v)
             val = np.array(result["constants"][v])
             for c, dv_dc in v.gradients.items():
-                with warnings.catch_warnings():  # skip pesky divide-by-zeros
-                    warnings.simplefilter("ignore")
+                with pywarnings.catch_warnings():  # skip pesky divide-by-zeros
+                    pywarnings.simplefilter("ignore")
                     dlogv_dlogc = dv_dc * result["constants"][c]/val
-                    before = self.v_ss.get(c, 0)
-                    self.v_ss[c] = before + dlogcost_dlogv*dlogv_dlogc
+                    gpv_ss[c] = gpv_ss.get(c, 0) + dlogcost_dlogv*dlogv_dlogc
                 if v in cost_senss:
-                    if c in self.cost.varkeys:
+                    if c in self.cost.vks:
                         dlogcost_dlogv = cost_senss.pop(v)
                         before = cost_senss.get(c, 0)
                         cost_senss[c] = before + dlogcost_dlogv*dlogv_dlogc
         result["sensitivities"]["cost"] = cost_senss
-        result["sensitivities"]["variables"] = KeyDict(self.v_ss)
+        result["sensitivities"]["variables"] = KeyDict(gpv_ss)
         result["sensitivities"]["constants"] = \
             result["sensitivities"]["variables"]  # NOTE: backwards compat.
-        result["soltime"] = solver_out["soltime"]
         return SolutionArray(result)
 
     def check_solution(self, cost, primal, nu, la, tol, abstol=1e-20):
@@ -388,6 +410,8 @@ class GeometricProgram:
                 raise Infeasible("Primal solution violates constraint: %s is "
                                  "greater than 1" % primal_exp_vals[mi].sum())
         # check dual sol #
+        if self.integersolve:
+            return
         # note: follows dual formulation in section 3.1 of
         # http://web.mit.edu/~whoburg/www/papers/hoburg_phd_thesis.pdf
         if not almost_equal(self.nu_by_posy[0].sum(), 1):
@@ -399,7 +423,7 @@ class GeometricProgram:
                 raise Infeasible("Dual solution has negative entries as"
                                  " large as %s." % minnu)
         if any(np.abs(A.T.dot(nu)) > tol):
-            raise Infeasible("Sum of nu^T * A did not vanish.")
+            raise Infeasible("Dual: sum of nu^T * A did not vanish.")
         b = np.log(self.cs)
         dual_cost = sum(
             self.nu_by_posy[i].dot(
@@ -428,8 +452,8 @@ def gen_meq_bounds(missingbounds, exps, meq_idxs):  # pylint: disable=too-many-l
                     n_lower.add((key, "lower"))
         # (consider x*y/z == 1)
         # for a var (e.g. x) to be upper bounded by this monomial equality,
-        #   - vars of the same sign/side (y) must be lower bounded
-        #   - AND vars of the opposite sign/side (z) must be upper bounded
+        #   - vars of the same sign (y) must be lower bounded
+        #   - AND vars of the opposite sign (z) must be upper bounded
         p_ub = n_lb = frozenset(n_upper).union(p_lower)
         n_ub = p_lb = frozenset(p_upper).union(n_lower)
         for keys, ub in ((p_upper, p_ub), (n_upper, n_ub)):
