@@ -10,6 +10,7 @@ from ..nomials import Variable
 from .gp import GeometricProgram
 from ..nomials import PosynomialInequality, Posynomial
 from .. import NamedVariables
+from ..small_scripts import appendsolwarning, initsolwarning
 
 
 EPS = 1e-6  # 1 +/- this is used in a few relative differences
@@ -45,11 +46,11 @@ class SequentialGeometricProgram:
     >>> gp.solve()
     """
     gps = solver_outs = _results = result = model = None
-    with NamedVariables("SGP"):
-        slack = Variable("PCCPslack")
+    with NamedVariables("RelaxPCCP"):
+        slack = Variable("C")
 
-    def __init__(self, cost, model, substitutions, *,
-                 use_pccp=True, pccp_penalty=2e2, checkbounds=True):
+    def __init__(self, cost, model, substitutions,
+                 *, use_pccp=True, pccp_penalty=2e2, **kwargs):
         self.pccp_penalty = pccp_penalty
         if cost.any_nonpositive_cs:
             raise InvalidPosynomial("""an SGP's cost must be Posynomial
@@ -66,7 +67,7 @@ class SequentialGeometricProgram:
         self.approxconstraints = []
         self.sgpvks = set()
         x0 = KeyDict(substitutions)
-        x0.varkeys = model.varkeys  # for string access and so forth
+        x0.vks = model.vks  # for string access and so forth
         for cs in model.flat():
             try:
                 if not hasattr(cs, "as_hmapslt1"):
@@ -82,7 +83,7 @@ class SequentialGeometricProgram:
                     constraint = (Posynomial(hmaplt1) <= self.slack)
                     constraint.generated_by = cs
                     self.approxconstraints.append(constraint)
-                    self.sgpvks.update(constraint.varkeys)
+                    self.sgpvks.update(constraint.vks)
         if not self.sgpconstraints:
             raise UnnecessarySGP("""Model valid as a Geometric Program.
 
@@ -91,13 +92,13 @@ Signomial Constraints, since Models without Signomials have global
 solutions and can be solved with 'Model.solve()'.""")
         self._gp = GeometricProgram(
             cost, self.approxconstraints + self.gpconstraints,
-            substitutions, checkbounds=checkbounds)
+            substitutions, **kwargs)
         self._gp.x0 = x0
         self.a_idxs = defaultdict(list)
-        cost_mons = self._gp.k[0]
-        sp_mons = sum(self._gp.k[:1+len(self.approxconstraints)])
+        last_cost_mon = self._gp.k[0]
+        first_gp_mon = sum(self._gp.k[:1+len(self.approxconstraints)])
         for row_idx, m_idx in enumerate(self._gp.A.row):
-            if cost_mons <= m_idx <= sp_mons:
+            if last_cost_mon <= m_idx <= first_gp_mon:
                 self.a_idxs[self._gp.p_idxs[m_idx]].append(row_idx)
 
     # pylint: disable=too-many-locals,too-many-branches,too-many-statements
@@ -149,7 +150,7 @@ solutions and can be solved with 'Model.solve()'.""")
                     "Unsolved after %s iterations. Check `m.program.results`;"
                     " if they're converging, try `.localsolve(...,"
                     " iteration_limit=NEWLIMIT)`." % len(self.gps))
-            gp = self.gp(x0, cleanx0=True)
+            gp = self.gp(x0, cleanx0=(len(self.gps) >= 1))  # clean the first x0
             self.gps.append(gp)  # NOTE: SIDE EFFECTS
             if verbosity > 1:
                 print("\nGP Solve %i" % len(self.gps))
@@ -169,13 +170,14 @@ solutions and can be solved with 'Model.solve()'.""")
             if prevcost is None:
                 continue
             rel_improvement = (prevcost - cost)/(prevcost + cost)
-            if cost*(1 - EPS) > prevcost + EPS and verbosity > -1:
+            if cost/prevcost >= 1 + 10*EPS:
                 pywarnings.warn(
-                    "SGP not convergent: Cost rose by %.2g%% on GP solve %i."
-                    " Details can be found in `m.program.results` or by"
-                    " solving at a higher verbosity. Note that convergence"
+                    "SGP not convergent: Cost rose by %.2g%% (%.6g to %.6g) on"
+                    " GP solve %i. Details can be found in `m.program.results`"
+                    " or by solving at a higher verbosity. Note convergence"
                     " is not guaranteed for models with SignomialEqualities."
-                    % (100*(cost - prevcost)/prevcost, len(self.gps)))
+                    % (100*(cost - prevcost)/prevcost,
+                       prevcost, cost, len(self.gps)))
                 rel_improvement = cost = None
         # solved successfully!
         self.result = gp.generate_result(solver_out, verbosity=verbosity-3)
@@ -186,29 +188,35 @@ solutions and can be solved with 'Model.solve()'.""")
             print("Solving took %.3g seconds and %i GP solves."
                   % (self.result["soltime"], len(self.gps)))
         if hasattr(self.slack, "key"):
+            initsolwarning(self.result, "Slack Non-GP Constraints")
             excess_slack = self.result["variables"][self.slack.key] - 1  # pylint: disable=no-member
-            if excess_slack <= EPS:
-                del self.result["freevariables"][self.slack.key]  # pylint: disable=no-member
-                del self.result["variables"][self.slack.key]  # pylint: disable=no-member
-                del self.result["sensitivities"]["variables"][self.slack.key]  # pylint: disable=no-member
-                slackconstraint = self.gpconstraints[0]
-                del self.result["sensitivities"]["constraints"][slackconstraint]
-            elif verbosity > -1:
-                pywarnings.warn(
-                    "Final solution let signomial constraints slacken by"
-                    " %.2g%%. Calling .localsolve with a higher"
-                    " `pccp_penalty` (it was %.3g this time) will reduce"
-                    " final slack if the model is solvable with less. If"
-                    " you think it might not be, check by solving with "
-                    "`use_pccp=False, x0=(this model's final solution)`.\n"
-                    % (100*excess_slack, self.pccp_penalty))
+            if excess_slack > EPS:
+                msg = ("Final PCCP solution let non-GP constraints slacken by"
+                       " %.2g%%." % (100*excess_slack))
+                appendsolwarning(msg, (1 + excess_slack), self.result,
+                                 "Slack Non-GP Constraints")
+                if verbosity > -1:
+                    print(msg +
+                          " Calling .localsolve(pccp_penalty=...) with a higher"
+                          " `pccp_penalty` (it was %.3g this time) will reduce"
+                          " slack if the model is solvable with less. To verify"
+                          " that the slack is needed, generate an SGP with"
+                          " `use_pccp=False` and start it from this model's"
+                          "  solution: e.g. `m.localsolve(use_pccp=False, x0="
+                          "m.solution[\"variables\"])`." % self.pccp_penalty)
+            del self.result["freevariables"][self.slack.key]  # pylint: disable=no-member
+            del self.result["variables"][self.slack.key]  # pylint: disable=no-member
+            del self.result["sensitivities"]["variables"][self.slack.key]  # pylint: disable=no-member
+            slackconstraint = self.gpconstraints[0]
+            del self.result["sensitivities"]["constraints"][slackconstraint]
         return self.result
 
     @property
     def results(self):
         "Creates and caches results from the raw solver_outs"
         if not self._results:
-            self._results = [o["generate_result"]() for o in self.solver_outs]
+            self._results = [gp.generate_result(s_o, dual_check=False)
+                             for gp, s_o in zip(self.gps, self.solver_outs)]
         return self._results
 
     def gp(self, x0=None, *, cleanx0=False):
@@ -216,7 +224,10 @@ solutions and can be solved with 'Model.solve()'.""")
         if not x0:
             return self._gp  # return last generated
         if not cleanx0:
-            x0 = KeyDict(x0)
+            cleanedx0 = KeyDict()
+            cleanedx0.vks = self._gp.x0.vks
+            cleanedx0.update(x0)
+            x0 = cleanedx0
         self._gp.x0.update({vk: x0[vk] for vk in self.sgpvks if vk in x0})
         p_idx = 0
         for sgpc in self.sgpconstraints:
